import {
  max,
  select,
  json,
  geoPath,
  geoTransverseMercator,
  zoom,
  event,
  scaleLinear,
  scaleLog,
  axisRight,
  range,
  format
} from 'd3';

const svg = select('svg');
var defs = svg.append("defs");

var gradient = defs.append("linearGradient")
   .attr("id", "svgGradient")
   .attr("x1", "0%")
   .attr("x2", "0%")
   .attr("y1", "0%")
   .attr("y2", "100%");

gradient.append("stop")
   .attr('class', 'start')
   .attr("offset", "0%")
   .attr("stop-color", "#490CA3")
   .attr("stop-opacity", 1);

gradient.append("stop")
   .attr('class', 'end')
   .attr("offset", "100%")
   .attr("stop-color", "#EEE8B4")
   .attr("stop-opacity", 1);
const githubRaw =
  'https://raw.githubusercontent.com/deldersveld/topojson/master/countries/india/india-states.jsonhttps://raw.githubusercontent.com/deldersveld/topojson/master/countries/india/india-states.json';
const provsRaw = "https://raw.githubusercontent.com/jeronimoek/corona-casos/master/PROVINCIAS_GEOJSON"

const width = +svg.attr('width');
const height = +svg.attr('height');
const margin = { top: 50, bottom: 50, left: 25, right: 25};

const projection = d3
  .geoTransverseMercator()
  .center([2.5, -38.5])
  .rotate([66, 0])
  .scale((height * 51) / 33) // (height*56.5)/33
  .translate([width / 2, height / 2 - 22]); // ..., height/2
const path = geoPath().projection(projection);

select('svg')
  .append('path')
  .attr('class', 'fondo')
  .attr('d', path({ type: 'Sphere' }));

const g = svg.append('g');

svg.call(
  zoom().on('zoom', () => {
    g.attr('transform', event.transform);
  })
);
const colorScale = scaleLog();

const colorValue = (d) => parseInt(d.properties.casos);

json(githubRaw).then((data) => {
  const values = [];

  data.objects['departamentos-argentina'].geometries.forEach((d) =>
    values.push(parseInt(d.properties.casos))
  );

  const maxValue = max(values);

  const domain = [1, maxValue];
  const dataRange = [height - margin.top - margin.bottom, +margin.top];
  const colorRange = ['#EEE8B4', '#490CA3'];

  colorScale.clamp(true).domain([1, maxValue]).range(colorRange);

  g.selectAll('path')
    .data(
      topojson.feature(data, data.objects['departamentos-argentina']).features
    )
    .enter()
    .append('path')
    .attr('class', 'dptos')
    .attr('d', path)
    .attr('fill', (d) => colorScale(colorValue(d)))
    .append('title')
    .text(
      (d) =>
        d.properties.departamento +
        ', ' +
        d.properties.provincia +
        '. Casos confirmados: ' +
        d.properties.casos
    );

  const logScale = scaleLog().domain(domain).range([height - margin.top - margin.bottom, +margin.top]);

  const xAxis = axisRight()
  	.scale(logScale)
  	.tickValues([1,10,100,1000,10000,20000])
  	.tickFormat((d)=>{
    	const x = d == 1000 ? "1k" : d == 1? "1" : format(".2s")(d)
      return x
  	})
    .tickSize(6, 0);
	
  svg.append("g").append("rect")
  	.attr("fill", "#ffe2e2")
  	.attr("stroke", "grey" )
  	.attr('x', 20)
  	.attr('y', margin.top-10)
  	.attr("width", 60)
  	.attr("height",dataRange[0]-dataRange[1]+20)
  	.attr("rx", 6)
  
  svg.append("g")
    .attr('transform', 'translate(' + (margin.left+20) + ',0)')
    .attr('class', 'xAxis')
    .call(xAxis);

  var num_colours = colorRange.length;
  var diff = dataRange[0] - dataRange[1];
	svg.append("g").append("rect")
  	.attr("stroke", "black" )
    .attr('x', 25)
    .attr('y', dataRange[1])
    .attr('height', diff+1)
    .attr('width', 20)
  	.attr("fill", "url(#svgGradient)");
});

/*  CONTORNO DE PROVINCIAS: (Tiene varios errores)
json(provsRaw).then(data => {

  g.selectAll('#path')
    .data(data.features)
    .enter().append('path')
  		.attr('id', (d)=>d.id)
      .attr('class', 'provs')
      .attr('d', path)
      .attr("fill","None")
});
*/